$dark-charcoal: #333333;
$skelly-bone-blue: #a4d6d9;
$cadet-skelly: cadetblue;
$laser-red: #d95158;
$disabled-laser-red: #8c5454;
$freemocap-green: #365d5f;
$freemocap-dark-green: #254342;
$button-blue: #29394a;
$tree-heading-blue: #5793cf;
$disabled-button-blue: #687e90;
$tree-primary: #7983b1;
$tree-secondary: #a3b1ed;
$plain-white: white;
$plain-black: black;

* {
    font-family: "Dosis", sans-serif;
    /*font-size: 12px;*/
    color: $dark-charcoal;
}

QGroupBox {
    border: 1px solid $freemocap-dark-green;
    border-radius: 4px;
    margin-top: 10px;
    font-weight: bold;
    background-color: $skelly-bone-blue;
}

QStatusBar {
    color: $freemocap-green;
}

QMainWindow {
    background-color: $freemocap-green;
}

QTabWidget::pane {
    border: 1px solid $plain-white;
    top: 1px;
    background: $cadet-skelly;
}

QTabBar::tab {
    border: 1px solid;
    padding: 5px;
    color: $freemocap-green;
    border-top-left-radius: 20px;
    border-top-right-radius: 5px;
    /*min-width: 200px;*/
}

QTabBar::tab {
    background: $plain-white;
    border-color: $freemocap-green;
    margin-bottom: -5px;
    font-weight: bolder;
    font-size: 16px;
}

QTabBar::tab:selected {
    background: $freemocap-green;
    color: $plain-white;
    border-color: $plain-white;
}

QPushButton {
    font-weight: bold;
    border-radius: 4px;
    border: 2px solid $button-blue;
    background-color: $button-blue;
    color: $plain-white;
}

QPushButton:disabled {
    background-color: $tree-primary;
    border: 1px solid $tree-primary;
    color: $skelly-bone-blue;
}

QPushButton:hover {
    border: 3px solid $plain-white;
}

QPushButton#start_recording_button {
    font-size: 18px;
    background-color: $laser-red;
}
QPushButton#start_recording_button:disabled {
    background-color: $disabled-laser-red;
}

QPushButton#stop_recording_button {
    font-size: 18px;

}
QPushButton#stop_recording_button:enabled {
    /*font-size: 24px;*/
    border: 3px solid $laser-red;
}

WelcomeScreenButton {
    border: 2px solid $dark-charcoal;
    background-color: $freemocap-green;
    color: $plain-white;
    font-size: 24px;
    border-radius: 20px;
    padding: 10px;
}

WelcomeScreenButton[recommended_next = true] {
    background-color: $laser-red;
    color: $plain-white;
}

QLabel {
    font-size: 14px;
    color: $dark-charcoal;

    QTreeView & {
        color: $plain-black;
  }
}

QLabel:disabled {
    color: $skelly-bone-blue
}

QLineEdit {
    border: 1px solid $button-blue;
    border-radius: 4px;
    padding: 0 8px;
    color: $dark-charcoal;
    background-color: $plain-white;
}
QLineEdit:disabled {
    color: $dark-charcoal;
    background-color: $disabled-button-blue;
}

QTreeView {
    background-color: $tree-primary;
    color: $plain-black;
    font-weight: bold;
    font-family: "Roboto", sans-serif;
    font-size: 12px;
}

QTreeView::item:has-children {
    background-color: $tree-heading-blue;
    color: $plain-white;
}

QTreeView::item:selected,
    QTreeView::item:hover{
    background-color: $plain-white;
    color: $dark-charcoal;
}

LogViewWidget {
    background-color: $freemocap-dark-green;
    color: $plain-white;
    font-family: "Roboto", sans-serif;
    font-size: 12px;
    padding: 8px;
}


